'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolveDep = resolveDep;
exports.useImplicitPrefix = useImplicitPrefix;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getImplicitPrefixes() {
  const babelConfigFile = _path2.default.join(process.cwd(), '.babelrc');
  if (!_fs2.default.existsSync(babelConfigFile)) {
    return null;
  }

  try {
    const { plugins = [] } = JSON.parse(_fs2.default.readFileSync(babelConfigFile, { encoding: 'utf8' }));
    const moduleResolver = plugins.find(item => {
      if (!Array.isArray(item)) {
        return false;
      } else {
        return item[0] === 'module-resolver';
      }
    });

    if (!moduleResolver) {
      return null;
    }

    return moduleResolver[1];
  } catch (e) {
    console.error('failed to read .babelrc, error: ', e.toString());
    return null;
  }
}

const MODULE_RESOLVER_CONF = getImplicitPrefixes() || {};
function resolveDep(dir, dep) {
  if ((0, _utils.isRelative)(dep)) {
    const suf = (0, _utils.getSuffix)(dep);
    const joined = _path2.default.join(dir, dep);
    if (!suf || suf === 'js') {
      return resolveFullpathJsModule(joined);
    } else {
      return joined;
    }
  }

  // 绝对路径导入都是用的require('相对路径')的形式
  let ret = '';

  ret = testWithRoot(dir, dep);
  if (ret) {
    return ret;
  }

  ret = testWithAlias(dir, dep);
  if (ret) {
    return ret;
  }

  return '';
}

function resolveFullpathJsModule(fullpath) {
  const isExists = _fs2.default.existsSync(fullpath);
  // fixme: 支持图片？
  const isDir = isExists ? _fs2.default.statSync(fullpath).isDirectory() : false;
  const addIndexJs = isDir && _path2.default.join(fullpath, 'index.js');
  if (isDir && _fs2.default.existsSync(addIndexJs)) {
    return addIndexJs;
  }

  const addJs = (0, _utils.addSuffix)(fullpath, 'js');
  if (_fs2.default.existsSync(addJs)) {
    return addJs;
  }

  return '';
}

const CURRENT_WD = process.cwd();
function testWithRoot(dir, dep) {
  const { root = [] } = MODULE_RESOLVER_CONF;
  for (let ii = 0; ii < root.length; ii++) {
    const item = root[ii];
    const tryPath = _path2.default.join(CURRENT_WD, _path2.default.join(item, dep));
    const resolveFullPathRes = resolveFullpathJsModule(tryPath);
    if (resolveFullPathRes) {
      return resolveFullPathRes;
    }
  }
  return '';
}

function testWithAlias(dir, dep) {
  const { alias = {} } = MODULE_RESOLVER_CONF;
  const firstPart = dep.split('/')[0];
  const hit = alias[firstPart];
  if (!hit) {
    return '';
  }

  const tryPath = _path2.default.join(CURRENT_WD, _path2.default.join(hit, dep));
  const resolveFullPathRes = resolveFullpathJsModule(tryPath);
  if (resolveFullPathRes) {
    return resolveFullPathRes;
  }

  return '';
}

function useImplicitPrefix(p) {
  let ret;
  ret = tryWithRoot(p);
  if (ret) {
    return ret;
  }

  ret = tryWithAlias(p);
  if (ret) {
    return ret;
  }
  throw new Error('no supposed to happen');
}

function tryWithRoot(p) {
  const { root = [] } = MODULE_RESOLVER_CONF;
  for (let ii = 0; ii < root.length; ii++) {
    const item = root[ii];
    const tryPath = (0, _utils.addTrailingSlash)(_path2.default.join(CURRENT_WD, item));
    if (_path2.default.normalize(p).startsWith(_path2.default.normalize(tryPath))) {
      return _path2.default.relative(tryPath, p);
    }
  }
}

function tryWithAlias(p) {
  const { alias = {} } = MODULE_RESOLVER_CONF;
  const keys = Object.keys(alias);
  for (let ii = 0; ii < keys.length; ii++) {
    const aliasName = keys[ii];
    const aliasValue = alias[aliasName];
    const tryPath = (0, _utils.addTrailingSlash)(_path2.default.join(CURRENT_WD, aliasValue));
    if (_path2.default.normalize(p).startsWith(_path2.default.normalize(tryPath))) {
      return _path2.default.join(aliasName, _path2.default.relative(tryPath, p));
    }
  }
}