'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addSuffix = addSuffix;
exports.getSuffix = getSuffix;
exports.trimSuffix = trimSuffix;
exports.isRelative = isRelative;
exports.addTrailingSlash = addTrailingSlash;
exports.isSuperDir = isSuperDir;
exports.toAbsolute = toAbsolute;
exports.isSameFile = isSameFile;
exports.depMatchFs = depMatchFs;
exports.fsToDep = fsToDep;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function addSuffix(dep, suf = 'js') {
  if (new RegExp('\\.' + suf + '$').test(dep)) {
    return dep;
  }
  return dep + '.' + suf;
}

function getSuffix(f = '') {
  const match = f.match(/\.([^/.]+)$/);
  return match && match[1] || '';
}

function trimSuffix(dep, suf = 'js') {
  const re = new RegExp('\\.' + suf + '$');
  return dep.replace(re, '');
}

function isRelative(file) {
  return (/^\.\/|\.\.\//.test(file)
  );
}

function addTrailingSlash(dir) {
  if (/\/$/.test(dir)) {
    return dir;
  }
  return `${dir}/`;
}

function isSuperDir(superDir, childDir) {
  return addTrailingSlash(childDir).startsWith(addTrailingSlash(superDir));
}

function toAbsolute(f, b) {
  if (_path2.default.isAbsolute(f)) {
    return f;
  }
  return _path2.default.join(b, f);
}

function isSameFile(f1, f2) {
  return _path2.default.normalize(f1).toLowerCase() === _path2.default.normalize(f2).toLowerCase();
}

const PARSE_FILE_NAME = /^(.*?)(@\d[xX])?\.([^.]+)$/;
function parseName(filename) {
  const match = filename.match(PARSE_FILE_NAME);
  return {
    basename: match[1],
    modifier: match[2],
    extension: match[3]
  };
}

function isPicture(type) {
  const low = type.toLowerCase();
  return low === 'png' || low === 'jpg' || low === 'gif';
}

function depMatchFs(depPath, fsPath) {
  if (isSameFile(addSuffix(depPath, 'js'), fsPath)) {
    return true;
  }
  const filenameFs = _path2.default.basename(fsPath);
  const filenameDep = _path2.default.basename(depPath);
  const dirnameFs = _path2.default.dirname(fsPath);
  const dirnameDep = _path2.default.dirname(depPath);

  if (isSameFile(dirnameFs, depPath) && filenameFs.toLowerCase() === 'index.js') {
    return true;
  }

  const { basename, modifier, extension } = parseName(filenameFs);
  if (isSameFile(dirnameFs, dirnameDep) && filenameDep === `${basename}.${extension}` && isPicture(extension)) {
    return true;
  }

  return isSameFile(depPath, fsPath);
}

function fsToDep(fsPath) {
  const filenameFs = _path2.default.basename(fsPath);
  if (filenameFs.toLowerCase() === 'index.js') {
    return _path2.default.dirname(fsPath);
  }

  const { basename, modifier, extension } = parseName(filenameFs);
  if (isPicture(extension)) {
    return _path2.default.join(_path2.default.dirname(fsPath), `${basename}.${extension}`);
  }

  return trimSuffix(fsPath, 'js');
}